class FcgParams:
    PATH = "PATH"
    CONTEXT_DOCUMENT_ROOT = "CONTEXT_DOCUMENT_ROOT"
    CONTEXT_PREFIX = "CONTEXT_PREFIX"
    CONTENT_TYPE = "CONTENT_TYPE"
    CONTENT_LENGTH = "CONTENT_LENGTH"
    DOCUMENT_ROOT = "DOCUMENT_ROOT"
    GATEWAY_INTERFACE = "GATEWAY_INTERFACE"
    HTTP_ACCEPT = "HTTP_ACCEPT"
    HTTP_ACCEPT_ENCODING = "HTTP_ACCEPT_ENCODING"
    HTTP_ACCEPT_LANGUAGE = "HTTP_ACCEPT_LANGUAGE"
    HTTP_CACHE_CONTROL = "HTTP_CACHE_CONTROL"
    HTTP_COOKIE = "HTTP_COOKIE"
    HTTP_CONNECTION = "HTTP_CONNECTION"
    HTTP_HOST = "HTTP_HOST"
    HTTP_USER_AGENT = "HTTP_USER_AGENT"
    HTTP_UPGRADE_INSECURE_REQUESTS = "HTTP_UPGRADE_INSECURE_REQUESTS"
    QUERY_STRING = "QUERY_STRING"
    REMOTE_ADDR = "REMOTE_ADDR"
    REQUEST_SCHEME = "REQUEST_SCHEME"
    REQUEST_URI = "REQUEST_URI"
    REMOTE_PORT = "REMOTE_PORT"
    REQUEST_METHOD = "REQUEST_METHOD"
    SCRIPT_NAME = "SCRIPT_NAME"
    SCRIPT_FILENAME = "SCRIPT_FILENAME"
    SERVER_ADDR = "SERVER_ADDR"
    SERVER_ADMIN = "SERVER_ADMIN"
    SERVER_NAME = "SERVER_NAME"
    SERVER_PORT = "SERVER_PORT"
    SERVER_PROTOCOL = "SERVER_PROTOCOL"
    SERVER_SIGNATURE = "SERVER_SIGNATURE"
    SERVER_SOFTWARE = "SERVER_SOFTWARE"
    UNIQUE_ID = "UNIQUE_ID"
    X_FORWARDED_FOR = "X-Forwarded-For"
    X_FORWARDED_PROTO = "X-Forwarded-Proto"
    X_FORWARDED_PORT = "X-Forwarded-Port"
